#include <mega8.h>
#include <math.h>
#include <alcd.h>
#include <delay.h>
#include <stdio.h>
#include <i2c.h>
#asm
    .equ __i2c_port=0x15
    .equ __sda_bit=4
    .equ __scl_bit=5
#endasm

#define DEVICE_address_write        0xEE
#define DEVICE_address_read         0xEF
#define PT_writeAddress_register    0xF4
#define PT_resultAddress_register   0xF6
#define T_control_register          0x2E
#define oss                         0
#define P_control_register          (0x34+(oss<<6))
#define P0                          101325
#define ACK                         1
#define NOACK                       0

#define     AC1_add     0xAA
#define     AC2_add     0xAC
#define     AC3_add     0xAE
#define     AC4_add     0xB0
#define     AC5_add     0xB2
#define     AC6_add     0xB4
#define     B1_add      0xB6
#define     B2_add      0xB8
#define     MB_add      0xBA
#define     MC_add      0xBC
#define     MD_add      0xBE

short int ac1,ac2,ac3,b1,b2,mb,md;
unsigned short int ac4,ac5,ac6;
long int mc;
long   UT,UP,T,P,x1,x2,x3,b3,b5,b6;
unsigned long int b4,b7;
unsigned char data[2];
int  n1,n2,temp_t1,temp_t2; 
char lcd_buf[17]; 

///fffffffff
////fffffffffffff


void write_to_device(unsigned char address_register ,unsigned char control_register)
  {
   i2c_start();
   i2c_write(DEVICE_address_write);
   i2c_write(address_register);
   i2c_write(control_register);
   i2c_stop();
   }

void read_from_device(unsigned char address_register)
  {
   i2c_start();
   i2c_write(DEVICE_address_write);
   i2c_write(address_register);
   i2c_start();
   i2c_write(DEVICE_address_read);
   data[0]=i2c_read(ACK); 
   data[1]=i2c_read(NOACK);
   i2c_stop();
  } 

int power(int i , int j)
 {
int k=0,r=1;
 for(k=0;k<j;k++){
    r=r*i;
    }
 return r;    
 }
 
void calibre(){
//********************************** CALIBRATION ************************************
read_from_device(AC1_add);
ac1=(short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"AC1= %d",ac1);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(AC2_add);
ac2=(short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"AC2= %d",ac2);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(AC3_add);
ac3=(short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"AC3= %d",ac3);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(AC4_add);
ac4=(unsigned short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"AC4= %d",ac4);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(AC5_add);
ac5=(unsigned short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"AC5= %d",ac5);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(AC6_add);
ac6=(unsigned short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"AC6= %d",ac6);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(B1_add);
b1=(short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"B1= %d",b1);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(B2_add);
b2=(short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"B2= %d",b2);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(MB_add);
mb=(short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"MB= %d",mb);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(MC_add);
mc=(long int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"MC= %d",mc);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
read_from_device(MD_add);
md=(short int)(data[0]<<8)+data[1];
lcd_clear();lcd_gotoxy(0,0);lcd_putsf("  CALIBRATION");
sprintf(lcd_buf,"MD= %d",md);lcd_gotoxy(0,1); lcd_puts(lcd_buf); delay_ms(100);
//...................................................................................
} 

void show_temp(){
        //*****---------calculating Temperature ---------******* 
        write_to_device(PT_writeAddress_register, T_control_register);  
        delay_ms(5);
        read_from_device(0xf6)  ;   
        UT=(long)((data[0]<<8)+data[1]);  
        x1=(UT-ac6)*ac5/(0x02<<14);
        x2= mc*(0x02<<10)/(x1+md) ;     // (0x02<<10) = 2 power 11
        b5=x1+x2;
        T=(b5+8)/(0x02<<3);
        n1=T/10;    n2=T%10;
        temp_t1=n1;
        temp_t2=n2;  
        lcd_clear();
        lcd_gotoxy(0,0);
        sprintf(lcd_buf,"T= %d.%d \xdfC",n1,n2); // lcd 
      
        lcd_puts(lcd_buf);
        }
void show_pre(){
        //********---------calculating Pressure ---------**********
        write_to_device(PT_writeAddress_register , P_control_register);
        delay_ms(5+power(3,oss));
        read_from_device(0xf6);
        UP=(long) ((data[0]<<8) +data[1]);    
        b6=b5-4000-1;         
        x1=(b2*(b6*b6/(0x02<<11)))/(0x02<<10);   
        x2=ac2*b6/(0x02<<10);             
        x3=x1+x2;            
        b3=(((ac1*4+x3) << oss) +2)/4;           
        x1=ac3*b6/(0x02<<12);    
        x2=(b1*(b6*b6/(0x02<<11)))/(0x02<<15);   
        x3=((x1+x2)+2)/(0x02<<1);                 
        b4=ac4*(unsigned long)(x3+32768)/(0x02<<14);      
        b7=((unsigned long)UP-b3)*(50000>>oss);       
        if(b7 < 0x80000000){P=(b7*2)/b4;}
        else {P=(b7/b4)*2;}               
        x1=(P/(0x02<<7))*(P/(0x02<<7));        
        x1=(x1*3038)/(0x02<<15);
        x2=(-7357*P)/(0x02<<15);
        P=P+(x1+x2+3791)/(0x02<<3);   
        n1=P/10000;     n2=P%10000;      
     sprintf(lcd_buf,"P= %d%d Pa",n1,n2); /// P is big num for that we / it to n1 n2..   
       lcd_gotoxy(0,1);
       lcd_puts(lcd_buf);
      // 
       } 
///end fffffff

void main(void)
{




lcd_init(16);
 

    
  lcd_clear();
  lcd_putsf("BMP180 "); 
  delay_us(100);  
  lcd_gotoxy(0,1);
  lcd_putsf("Pressure/Temp...");
  delay_ms(400);


i2c_init();
calibre();
  
 
while(1)
    {    
        
     show_temp();
     show_pre();   
     delay_ms(1000); 
    }   
}

       

